from ezaws import Region, RDS, DBEngine, DBInstanceType, ParameterStore
from pprint import pprint


if __name__ == "__main__":

    ps = ParameterStore(region="eu-central-1")
    db_pass = ps.get_parameter("/passwords/infrastructure/rds/pipeline01/password")
    db_user = ps.get_parameter("/passwords/infrastructure/rds/pipeline01/username")

    rds = RDS(
        region=Region.eu_central_1,
        master_user_password=db_pass.Parameter.Value,
        master_username=db_user.Parameter.Value,
        storage=5,
        db_name="pipeline01",
        db_engine=DBEngine.MYSQL.value,
        db_instance=DBInstanceType.db_t2_micro.value,
    )

    resp = rds.start_db()
    pprint(resp)



from ezaws import Log
import time
from ezaws.models.cloudwatch import (
    CreateLogGroupResponse,
    DeleteLogGroupResponse,
    Events,
    LogResponse,
    LogEvent,
    TailLogResponse,
)


def test_create_log_group():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    resp = cw_log.create_log_group()
    assert isinstance(resp, CreateLogGroupResponse)


def test_create_stream():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    cw_log.create_stream(stream_name="general")
    assert "general" in cw_log.streams.keys()


def test_log_events():
    """Log a few events"""
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    events = [
        {
            "timestamp": int(round(time.time() * 1000)),
            "message": "test_log_events",
        },
        {
            "timestamp": int(round(time.time() * 1000)),
            "message": "test_log_events",
        },
        LogEvent(message="tidbit", timestamp=int(round(time.time() * 1000))),
    ]
    resp = cw_log.log_events(events=events)
    assert isinstance(resp, LogResponse)


def test_log():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    resp = cw_log.log(message="test_log")
    assert isinstance(resp, LogResponse)


def test_log_messages():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    resp = cw_log.log_messages(
        messages=["test_log_messages", "test_log_messages"],
    )
    assert isinstance(resp, LogResponse)


def test_get_log_events():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    resp_list = []
    resp_list.append(cw_log.get_log_events_last_seconds(seconds=5))
    resp_list.append(cw_log.get_log_events_last_minutes(minutes=5))
    resp_list.append(cw_log.get_log_events_last_hours(hours=5))
    resp_list.append(cw_log.get_log_events_last_days(days=5))
    for resp in resp_list:
        assert isinstance(resp, Events)


def test_tail_log():
    """test the tail_log method as well as the reception of previously
    generated logs."""
    cw_log = Log(name="testing_log_group", region="eu-central-1")

    resp = cw_log.tail_log(n=200)
    assert isinstance(resp, TailLogResponse)
    messages = []
    for event in resp:
        messages.append(event.message)
    print(messages)
    # time.sleep(3)
    assert "test_log_events" in messages
    assert "test_log_messages" in messages
    assert "test_log" in messages


def test_delete_log_group():
    cw_log = Log(name="testing_log_group", region="eu-central-1")
    resp = cw_log.delete_log_group()
    assert isinstance(resp, DeleteLogGroupResponse)



from ezaws import Messenger
from ezaws import Message
from pprint import pprint
from ezaws import Region

if __name__ == "__main__":

    # instantiate the messanger
    msgnr = Messenger.from_queue_name(
        queue_name="example-queue", region=Region.eu_central_1
    )

    # send a few messages
    msg = msgnr.send_message("message 1")
    msg = msgnr.send_message("message 2")
    msg = msgnr.send_message("message 3")
    msg = msgnr.send_message("message 4")
    msg = msgnr.send_message("message 5")
    pprint(msg)

    # read and delete messages:
    msg_read = msgnr.read_messages(nr_of_messages=10)

    for m in msg_read:
        m = Message(**m)
        print(f"read message {m.Body}")
        resp = msgnr.delete_message(receipt_handle=m.ReceiptHandle)



from pprint import pprint
import csv
from ezaws import Region
from ezaws import DynamoDB

MAX = 200

if __name__ == "__main__":
    ddb = DynamoDB(region=Region.eu_central_1)
    result = ddb.scan(table_name="Humans")
    pprint(result)
    items = [
        {"id": 1, "name": "Said van de Klundert", "age": 38},
        {
            "id": 2,
            "name": "Jan van de Klundert",
        },
        {
            "id": 3,
            "name": "Marie van de Klundert",
        },
        {
            "id": 4,
            "name": "Anne van de Klundert",
        },
    ]
    ddb.put_items(table_name="Humans", items=items)
    ddb.put_item(table_name="Humans", item={"id": 5, "name": "Henk"})

    with open("data/names.csv", "rt") as f:
        csv_data = csv.DictReader(f)

        for row in csv_data:
            print(row)
            if MAX:
                ddb.put_item(
                    table_name="Humans",
                    item={"id": int(row["id"]), "name": row["name"]},
                )
                MAX -= 1
            else:
                break
    result = ddb.scan(table_name="Humans")
    pprint(result)



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB

if __name__ == "__main__":

    ddb = DynamoDB(region=Region.eu_central_1)
    result = ddb.list_tables()

    pprint(result)

    result = ddb.list_table_names()
    pprint(result)



from ezaws import S3, S3Bucket, Region, TCBuilder


if __name__ == "__main__":
    from pprint import pprint

    s3 = S3()
    resp = s3.list_buckets()
    bucket = S3Bucket(region=Region.eu_central_1, name="s3examplebucketusingezaws")

    bucket.create_bucket()

    resp = bucket.get_versioning()
    pprint(resp.dict())
    resp = bucket.upload_file("example.txt", "some_file")
    import time

    print("UPLOAD DONE")
    time.sleep(2)
    print("upload_file", resp)
    resp = bucket.delete_object(s3_key_name="some_file")
    pprint(resp)

    print("UPLOADING PUT OBJECT")
    resp = bucket.put_object(file_name="example.txt")
    pprint(resp)

    resp = bucket.get_object_metadata(s3_file_name="example.txt")
    print("HEAD OBJECT")
    pprint(resp)
    resp = bucket.get_file_size(s3_file_name="example.txt")
    print("FILESIZE", resp)
    resp = bucket.get_file_size(s3_file_name="example.txt", block_size="bytes")
    print("FILESIZE", resp)

    resp = bucket.delete_object(s3_key_name="example.txt")
    pprint(resp)
    print("CONTROLLED UPLOAD")
    tcbuilder = TCBuilder(use_threads=True, max_concurrency=5)
    resp = bucket.controlled_upload(file_name="example.txt", tc=tcbuilder)
    pprint(resp)
    resp = bucket.delete_object(s3_key_name="example.txt")
    pprint(resp)
    bucket.delete_bucket()



from ezaws import Messenger
from pprint import pprint
from ezaws import Region

if __name__ == "__main__":

    msgnr = Messenger.from_queue_name(
        queue_name="example-queue", region=Region.eu_central_1
    )

    msg = msgnr.send_message("message 1")

    pprint(msg)
    q_count = msgnr.get_queue_count()
    print(q_count)



from ezaws import ParameterStore
from ezaws.models.ssm import GetParameterResponse, RegionParameters

"""
    ps = ParameterStore(region="eu-central-1")
    param_resp = ps.get_parameter("/passwords/infrastructure/ssot_token")
    pprint(param_resp)
    region_param_resp = ps.get_region_parameters()
    pprint(region_param_resp)
"""


def test_get_parameter():
    ps = ParameterStore(region="eu-central-1")
    param_resp = ps.get_parameter("/passwords/infrastructure/ssot_token")
    assert isinstance(param_resp, GetParameterResponse)


def test_describe_region_parameters():
    ps = ParameterStore(region="eu-central-1")
    region_param = ps.describe_region_parameters()
    assert isinstance(region_param, RegionParameters)



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB, Table
import logging

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.INFO)

if __name__ == "__main__":

    table = Table(table_name="Humans", rcu=1, wcu=1)
    table.add_attribute(attribute_name="id", attribute_type="N", key_type="HASH")
    table.add_attribute(attribute_name="name", attribute_type="S", key_type="RANGE")
    ddb = DynamoDB(region=Region.eu_central_1)
    table_response = ddb.create_table(table=table)
    pprint(table_response)



from ezaws import Log
from pprint import pprint
import datetime

if __name__ == "__main__":

    cw_log = Log(name="cloudwatch_example", region="eu-central-1")

    resp = cw_log.tail_log(10)
    pprint(resp)
    resp = cw_log.tail_log()
    pprint(resp)
    for log in resp:
        print(log.message)
        print(log.timestamp)
        sec = log.timestamp / 1000.0
        print(datetime.datetime.utcfromtimestamp(sec))



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB


if __name__ == "__main__":

    ddb = DynamoDB(region=Region.eu_central_1)
    result = ddb.get_item(table_name="Humans", item={"id": 99, "name": "Arnetta"})

    pprint(result)



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB


if __name__ == "__main__":

    ddb = DynamoDB(region=Region.eu_central_1)
    result = ddb.scan(table_name="Humans")

    pprint(result)
